
#include "rhis.h"
#include "rhtree.h"

//Entry functions

int Entry::operator==(const Entry &) const
{
 	return TRUE;

}

double Entry::Area() const
{
	return 0.0;
}

int Entry::PrintEntry(ostream& os)const
{
	return 0;
}

//Used for storing the data and directory records...
RecordEntry::RecordEntry()
{
	int i;

	for (i=0; i < NUM_DIM ; i++)
	{
	 	m_low[i] = DOUBLE_MAX;
		m_high[i] = 0.0;
	}
	m_ins = -1L;
	m_del = -1L;
	m_info = RWNIL;
}

int RecordEntry::WriteRecord(RWFileManager* pFM) const
{
	int i;

	for (i=0; i< NUM_DIM; i++)
	{
		pFM->Write(m_low[i]);
		pFM->Write(m_high[i]);
	}
	pFM->Write(m_ins);
	pFM->Write(m_del);
	pFM->Write(m_info);
	return 0;
}

int RecordEntry::ReadRecord(RWFileManager* pFM)
{
	int i;

	for (i=0; i< NUM_DIM; i++)
	{
		pFM->Read(m_low[i]);
		pFM->Read(m_high[i]);
	}
	pFM->Read(m_ins);
	pFM->Read(m_del);
	pFM->Read(m_info);
	return 0;
}

int RecordEntry::GetSize()
{
	return NUM_DIM * (sizeof (m_low[0]) + sizeof (m_high[0])) +
			sizeof(m_ins) + sizeof(m_del) + sizeof(m_info);
}

int RecordEntry::operator==(const RecordEntry& record) const
{
	int i;
	
	for(i=0; i < NUM_DIM; i++)
	{
	 	if (m_low[i] != record.m_low[i])
			return FALSE;

		if (m_high[i] != record.m_high[i])
			return FALSE;
	}

	if ((m_ins != record.m_ins) || (m_del != record.m_del))
		return FALSE;

	return TRUE;
}

// Used to search for an entry

int RecordEntry::IdenticalEntry(const RecordEntry& record) const
{
	int i;
	
	for(i=0; i < NUM_DIM; i++)
	{
	 	if (m_low[i] != record.m_low[i])
			return FALSE;

		if (m_high[i] != record.m_high[i])
			return FALSE;
	}

	return TRUE;
}

int RecordEntry::IsAlive(long lTime)
{

 	//Find the latest alive one.
 	if ((-1L == lTime) && (-1L == m_del))
		return TRUE;

	if ((lTime >= m_ins) && ((-1L == m_del) || (m_del > lTime)))
		return TRUE;

	return FALSE;
}

double RecordEntry::Area() const
{
	int i;
	double area = 1.0;

	for (i=0; i < NUM_DIM; i++)
	{
		area *= (m_high[i] - m_low[i]);
	}
	return area;
}

// Sum of the length of all edges
double RecordEntry::Margin() const
{
	double fMargin = 0.0;
	int i;
	for (i=0; i < NUM_DIM; i++)
	{
		fMargin += (m_high[i] - m_low[i]);
	}
	return fMargin;
}

//Overlap Area of the entry with another entry.
double RecordEntry::OverlapArea(const RecordEntry &record) const
{
	RecordEntry NewRecord;
	int i;

	for (i=0; i < NUM_DIM; i++)
	{
		NewRecord.m_low[i] = MAX(m_low[i], record.m_low[i]);
		NewRecord.m_high[i] = MIN(m_high[i], record.m_high[i]);
		
		// Make sure if there is no overlap we fix the values
		if (NewRecord.m_low[i] > NewRecord.m_high[i])
			NewRecord.m_high[i] = NewRecord.m_low[i];
	}

	return (NewRecord.Area());
}

// Upon incoporating this entry.
double RecordEntry::AreaIncrease(const RecordEntry &record) const
{
	RecordEntry NewRecord;
	int i;

	for (i=0; i < NUM_DIM; i++)
	{
		NewRecord.m_low[i] = MIN(m_low[i], record.m_low[i]);
		NewRecord.m_high[i] = MAX(m_high[i], record.m_high[i]);
	}

	return (NewRecord.Area() - Area());
}

// See if the record is completely included or not.
int RecordEntry::DoesInclude(const RecordEntry &record) const
{
	int i;

	for (i=0; i < NUM_DIM; i++)
	{
	  if ((m_low[i] > record.m_low[i]) || (m_high[i] < record.m_high[i]))
	    return FALSE;
	}

	return TRUE;
}

int RecordEntry::DoesIncludeAndIsAlive(const RecordEntry &record) const
{
  // marioh: DO NOT modify the = signs. The tree will not work with
  // any other combination !??!!!?!?! %#@%!#@
  
	if (!DoesInclude(record))
		return FALSE;

	// Special cases first for the time.
	if (-1L == m_del)
	{
	 	// If both are alive
	 	if (-1L == record.m_del)
			return TRUE;
		else
      // DO NOT TOUCH THE EQUALS SIGN.
			return record.m_del >= m_ins;

	} else if (-1L == record.m_del)
	{
	  return m_del >= record.m_ins;
	} else
	{
	  if ((m_ins >= record.m_del) || (m_del <= record.m_ins))
	    return FALSE;
	}

	return TRUE;
}

// To check if there is an overlap.
int RecordEntry::Overlaps(const RecordEntry &record) const
{
	int i;

	for (i=0; i < NUM_DIM; i++)
	{
	  if ((m_low[i] >= record.m_high[i]) || (m_high[i] <= record.m_low[i]))
	    return FALSE;
	}

	return TRUE;
}

// To check if there is an overlap and also in time.
int RecordEntry::OverlapsAndIsAlive(const RecordEntry &record) const
{
  // marioh: DO NOT modify the = signs. The tree will not work with
  // any other combination !??!!!?!?! %#@%!#@
  
	if (!Overlaps(record))
		return FALSE;

	// Special cases first for the time.
	if (-1L == m_del)
	{
	 	// If both are alive
	 	if (-1L == record.m_del)
			return TRUE;
		else
      // DO NOT TOUCH THE EQUALS SIGN.
			return record.m_del >= m_ins;
		
	} else if (-1L == record.m_del)
	{
	  return m_del >= record.m_ins;
	} else
	{
	  if ((m_ins >= record.m_del) || (m_del <= record.m_ins))
			return FALSE;
	}

	return TRUE;
}

//Adjusted Entry after incorporating the record
RecordEntry RecordEntry::ReturnAdjustedEntry(const RecordEntry &record) const
{
	RecordEntry NewRecord = (*this);
	int i;
	
	for (i=0; i < NUM_DIM; i++)
	{
		NewRecord.m_low[i] = MIN(m_low[i], record.m_low[i]);
		NewRecord.m_high[i] = MAX(m_high[i], record.m_high[i]);
	}
	return NewRecord;
}

//Cover the Entry after incorporating the record
RecordEntry &RecordEntry::CoverEntry(const RecordEntry &record)
{
	int i;
	
	for (i=0; i < NUM_DIM; i++)
	{
		m_low[i] = MIN(m_low[i], record.m_low[i]);
		m_high[i] = MAX(m_high[i], record.m_high[i]);
	}
	return (*this);
}

int RecordEntry::ClearEntry()
{
	int i;
	for (i=0; i< NUM_DIM; i++)
	{
		m_low[i] = DOUBLE_MAX;
		m_high[i] = 0.0;
	}
	m_ins = -1;
	m_del = -1;
	m_info = RWNIL;
	return 0;
}

int RecordEntry::PrintEntry(ostream& os) const
{
	int i;
	for (i=0; i < NUM_DIM; i++)
	{
		os << m_low[i] << " ";
		os << m_high[i] << " ";
	}
	os << " " << m_ins << " " << m_del << endl;
	//os << m_info << endl;

	return 0;
}

