
// The main file for managing the rtree history.

#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <fstream.h>
#include <rw/cstring.h>
#include <rw/rwfile.h>
#include "rhis.h"
#include "rhtree.h"

#define INPUT_FILE	"t.txt"
#define	RH_DATABASE	"t.rht"

//Declaration of the stats variable
CStats stats;
CGeneralStats genStats;

int
main(int argc, char* argv[])
{
	int i;
	int	count;
	CStats insStats, delStats, queryStats;
	CQueryProfileStats queryProfileStats;
	RWCString sDataFile = INPUT_FILE;
	RWCString sDBName = RH_DATABASE;

	// Initialize the random seed
	srand( (unsigned)time( NULL ) );

	for (i=1; i < argc; i++)
	{
		if (!strcmp(argv[i], "-i"))
			sDataFile = argv[++i];
		else if (!strcmp(argv[i], "-d"))
			sDBName = argv[++i];
		else if (!strcmp(argv[i], "-h"))
		{
		 	cout << "Usage: " << argv[0] << " [-i Query_file]"
		 		<< " [-d database name] [-h] [-debug]"
				<< " [-queryprofile queryblocksize] [-nocache/-cache]"
				<< endl;
			return 0;
		} else if (!strcmp(argv[i], "-querydebug"))
		{
			RHTree::bQueryDebug = TRUE;
		} else if (!strcmp(argv[i], "-debug"))
		{
			RHTree::bQueryDebug = TRUE;
			RHTree::bDeleteDebug = TRUE;
		} else if (!strcmp(argv[i], "-nocache"))
		{
			RHTree::bCacheEnabled = FALSE;
		} else if (!strcmp(argv[i], "-cache"))
		{
			RHTree::bCacheEnabled = TRUE;
		} else if (!strcmp(argv[i], "-queryprofile"))
		{
			RHTree::m_bQueryProfileEnabled = TRUE;
			RHTree::m_nQueryProfileBlockSize = atoi(argv[++i]);
		} else
		{
		 	advise("Invalid argument");
			return 1;
		}
	}

	cout << "Starting execution." << endl << "Query file = " << sDataFile << " Database = " <<
		sDBName << endl << " Number of Dimensions = " << NUM_DIM << endl;

	// Check out the RHTree choices
	RHTree::PrintRHTreeChoices(cout);

	RHTree rhtree;
	rhtree.OpenTree(sDBName);

	// input file
	fstream iFile(sDataFile, ios::in);

	if ((iFile.fail()) || (iFile.eof()))
	{
	 	advise("main(): Unable to open the file.");
		return 1;
	}
	RecordEntry record;

	double fUniverse, fTrials, fTmp;
	int nOpType, nRetVal;
	long lCurTime, lOid;

	count = 0;

	// This are not used though but are there in the file.
	// iFile >> fUniverse >> fTrials >> fTmp >> fTmp >> fTmp;

	iFile >> lCurTime;

	while (-1 != lCurTime)
	{

		count++;

		iFile >> nOpType;
		iFile >> lOid;
		for (i=0; i < NUM_DIM; i++)
		{
			iFile >> record.m_low[i];
			iFile >> record.m_high[i];
		}

		// These are used for the record.
		iFile >> record.m_ins;
		iFile >> record.m_del;

		// cout << "Operation: " << nOpType << " Cur Time: " << lCurTime << " ";
		// cout << "Operation  " << nOpType << "  " ;
		// record.PrintEntry(cout);

		switch(nOpType)
		{
			case OP_QUERY:
				{
					int bDone = FALSE;
					stats = queryStats;
					if (RHTree::m_bQueryProfileEnabled)
						queryProfileStats.RememberStats(genStats, stats);

					// We want to find all the records which are
					// alive at m_del actually so increment it by one
					// otherwise the records which are inserted at
					// m_del are going to be missed.
					// record.m_del++;
					nRetVal = rhtree.QueryRecords(record,
						&(RHTree::QueryDirectoryRecordFunction),
						&(RHTree::QueryDataRecordFunction), TRUE, bDone);
					genStats.IncNumQueries();
					if (!rhtree.bCacheEnabled)
						rhtree.FlushCache();

					if (RHTree::m_bQueryProfileEnabled)
						queryProfileStats.UpdateProfileStats(genStats, stats);
					queryStats = stats;
					break;
				}
		 	default:
				advise("main(): Invalid operation type.");
				return 1;
		}

		if (0 != nRetVal)
		{
		 	advise("main(): Invalid return type from the operation.");
			return 1;
		}


		iFile >> lCurTime;

		if (iFile.eof())
		{
			advise("main(): unexpected EOF for the input file.");
			return 1;
		}

		if (count % 20000 == 0) {
			cout << " i =  " << count << endl;
		}
	}
	rhtree.CloseTree();

	queryStats.PrintStats(cout, "Query ", genStats.NumberOfQueries());
	cout << endl;
	genStats.PrintGeneralStats(cout);
	cout << endl;
	if (RHTree::m_bQueryProfileEnabled)
		queryProfileStats.PrintStats(cout);
	cout << endl;

	cout << "Quiting The program." << endl;

	return 0;
}
