#ifndef __RWMUTEX_H__
#define __RWMUTEX_H__

/*
 * Declarations for class RWMutex
 *
 * $Id: mutex.h,v 1.6 1993/09/10 02:56:53 keffer Exp $
 *
 ****************************************************************************
 *
 * Rogue Wave Software, Inc.
 * P.O. Box 2328
 * Corvallis, OR 97339
 * Voice: (503) 754-3010	FAX: (503) 757-6650
 *
 * Copyright (C) 1989, 1990, 1991, 1993.
 * This software is subject to copyright protection under the laws of
 * the United States and other countries.
 *
 ***************************************************************************
 *
 * This class is a portable implementation of a simple mutex lock
 * to be used for synchronizing multiple threads within a single process.
 * It is not suitable for use among threads of different processes.
 * 
 ***************************************************************************
 *
 * $Log: mutex.h,v $
 * Revision 1.6  1993/09/10  02:56:53  keffer
 * Switched RCS idents to avoid spurious diffs
 *
 * Revision 1.5  1993/08/04  21:17:22  jims
 * Change RW_WIN32_API to __WIN32__
 *
 * Revision 1.4  1993/07/29  06:42:17  jims
 * Change RW_WIN32_API to __WIN32__
 *
 * Revision 1.3  1993/07/09  07:55:49  jims
 * Port to Windows NT
 *
 * Revision 1.2  1993/04/15  20:35:42  keffer
 * Changed comments.
 *
 */

#ifdef RW_MULTI_THREAD		/* This class only relevant in MT situation */

#if defined(sun)		/* assuming Solaris 2.1 */
#  include <synch.h>
   typedef mutex_t RWMutexType;

#elif defined(__WIN32__)
#  include <windows.h>
   typedef HANDLE RWMutexType;

// defines for OS/2, et. al. go here
#else
#  error Class RWMutex is not yet supported in this environment
#endif

class RWMutex {
  RWMutexType mutex;
public:
  RWMutex();             // Initialize the mutex
  ~RWMutex();            // Destroy the mutex
  void acquire();        // Block until mutex released, acquire and continue
  void release();        // release the mutex
};

class RWGuard {
  RWMutex& rwmutex;
public:
  RWGuard(RWMutex& m) : rwmutex(m) { rwmutex.acquire(); }
  ~RWGuard() { rwmutex.release(); }
};

/*
 * =========================
 * RWMutex member functions
 * -------------------------
 */

inline RWMutex::RWMutex()  
{ 
# if defined(sun)
  mutex_init(&mutex, USYNC_THREAD, NULL); 
# elif defined(__WIN32__)
  mutex = CreateMutex(NULL, FALSE, NULL);
# endif
}

inline RWMutex::~RWMutex() 
{ 
# if defined(sun)
  mutex_destroy(&mutex); 
# elif defined(__WIN32__)
  CloseHandle(mutex);
# endif
}

inline void RWMutex::acquire() 
{ 
# if defined(sun)
  mutex_lock(&mutex);    
# elif defined(__WIN32__)
  WaitForSingleObject(mutex,INFINITE);
# endif
}

inline void RWMutex::release()
{ 
# if defined(sun)
  mutex_unlock(&mutex);  
# elif defined(__WIN32__)
  ReleaseMutex(mutex);
# endif
}

#endif  /* __WIN32__ */
#endif  /* __RWMUTEX_H__ */


