/*
 * RWInstanceManager
 *
 * $Id: instmgr.cpp,v 1.5 1993/09/10 03:59:57 keffer Exp $
 *
 ****************************************************************************
 *
 * Rogue Wave Software, Inc.
 * P.O. Box 2328
 * Corvallis, OR 97339
 *
 * Copyright (C) 1991, 1992.
 * This software is subject to copyright protection under the laws of
 * the United States and other countries.
 *
 ***************************************************************************
 *
 * $Log: instmgr.cpp,v $
 * Revision 1.5  1993/09/10  03:59:57  keffer
 * Added RW_RCSID macro.
 *
 * Revision 1.4  1993/08/05  11:40:24  jims
 * Remove destructor; instmgr for WIN16 DLLs only
 *
 * Revision 1.3  1993/07/29  06:42:17  jims
 * Change RW_WIN32_API to __WIN32__
 *
 * Revision 1.2  1993/07/09  08:04:10  jims
 * Change FAR to rwfar; Port to Windows NT
 *
 * Revision 1.1  1993/04/12  12:03:21  jims
 * Initial revision
 *
 * Revision 2.3  1993/02/05  07:36:14  jims
 * Now uses rwtsd.dll (Rogue Wave Task Specific Data) library
 *
 * Revision 2.2  1992/11/20  18:51:56  keffer
 * DEBUG -> RWDEBUG
 *
 * Revision 2.1  1992/11/16  04:23:53  keffer
 * Reflects changes to RWModel
 *
 * Revision 2.0  1992/10/23  03:36:49  keffer
 * RCS Baseline version
 *
 * 
 *    Rev 1.4   07 Jul 1992 16:22:22   KEFFER
 * Added additional preconditions.
 * 
 *    Rev 1.3   22 Jun 1992 17:54:24   KEFFER
 * Instance manager now uses fallback position if no process has been registered
 * 
 *    Rev 1.2   18 Mar 1992 11:50:22   KEFFER
 * 
 *    Rev 1.1   18 Mar 1992 11:44:42   KEFFER
 * Changed nil to rwnil.
 * 
 *    Rev 1.0   12 Nov 1991 13:53:22   keffer
 * Initial revision.
 */

#include "rw/instmgr.h"

RW_RCSID("Copyright (C) Rogue Wave Software --- $RCSfile: instmgr.cpp,v $ $Revision: 1.5 $ $Date: 1993/09/10 03:59:57 $");

/****************************************
 *					*
 *   Win16-DLL / Multi-thread CODE	*
 *					*
 ****************************************/

#if (defined(__DLL__) && defined(__WIN16__)) || defined(RW_MULTI_THREAD)

void rwfar*
RWInstanceManager::addValue()
{
#ifdef RWDEBUG
  assert(currentValue()==rwnil);
#endif

  void rwfar* value = newValue();	 // init value provided by specializing class

#if defined(__DLL__) && defined(__WIN16__)
  RWSetTaskSpecificData(tsd_key, value);

#elif defined(sun)
  thr_setspecific(tsd_key, value);

#elif defined(__WIN32__)
  TlsSetValue(tsd_key, value);

#endif 
  return value;
}

void rwfar*
RWInstanceManager::currentValue()
{
#if defined(__DLL__) && defined(__WIN16__)
  return RWGetTaskSpecificData(tsd_key); 

#elif defined(sun)
  void *value;
  thr_getspecific(tsd_key, &value);
  return value;

#elif defined(__WIN32__)
  return TlsGetValue(tsd_key);

#endif
}

void
RWInstanceManager::freeValue()
{
  deleteValue(currentValue());	  // give value to specializing class to delete

// Now make sure the next call to currentValue() returns NULL:
#if defined(__DLL__) && defined(__WIN16__)
  RWReleaseTaskSpecificData(tsd_key);

#elif defined(sun)
  thr_setspecific(tsd_key, NULL); 

#elif defined(__WIN32__)
  TlsSetValue(tsd_key, NULL);
#endif
}

RWInstanceManager::RWInstanceManager()
{
// Get a key -- Thread lib should init each thread to have NULL associated
//              with this key.

#if defined(__DLL__) && defined(__WIN16__)
  tsd_key = RWGetTaskSpecificKey();

#elif defined(sun)
  thr_keycreate(&tsd_key, NULL);

#elif defined(__WIN32__)
  tsd_key = TlsAlloc();
#endif
}

#else	/* Neither Win16-DLL nor Mulit-threaded */

#  error RWInstanceManager only to be used in Win16-DLL or MT Envirnonments

#endif	/* (__DLL__ &&__WIN16) || RW_MULTI_THREAD */
