// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Outputs all boxes alive at the given time instant in gnuplot or box format.
*/

#include "global.h"

int main(int argc, char** argv)
{
	if (argc != 4)
	{
		cerr << "Usage: " << argv[0] << " box_file time_instant output_type (gnuplot|boxes)." << endl;
		return -1;
	}

	int snapshotTime = atoi(argv[2]);
	int output = (strcmp(argv[3], "boxes") == 0) ? 1 : 0;

	ifstream bf(argv[1]);
	if (! bf)
	{
		cerr << "Cannot open file " << argv[1] << "." << endl;
		return -1;
	}

	char buf[1000];

	while (bf.getline(buf, 1000))
	{
		while (bf && strlen(buf) == 0) bf.getline(buf, 1000);
		if (! bf) break;

		istringstream ss(buf);
		int id, t1, t2;
		double x1, x2, y1, y2;

		ss >> id >> x1 >> x2 >> y1 >> y2 >> t1 >> t2;
		if (! bf.good()) break;

		if (t1 <= snapshotTime && snapshotTime <= t2)
		{
			if (output == 0)
			{
				cout << x1 << " " << y1 << endl;
				cout << x2 << " " << y1 << endl;
				cout << x2 << " " << y2 << endl;
				cout << x1 << " " << y2 << endl;
				cout << x1 << " " << y1 << endl;
				cout << endl << endl;
			}
			else
			{
				cout << id << " " << x1 << " " << x2 << " " << y1 << " " << y2 << " " << snapshotTime << " " << snapshotTime << endl;
			}
		}
	}
}
