// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Converts all boxes to gnuplot format for visualization purposes, using all possible projections.
*/

#include "global.h"

int main(int argc, char** argv)
{
	if (argc != 3)
	{
		cerr << "Usage: " << argv[0] << " input_file projection (xt | yt | xy | xyt)." << endl;
		return -1;
	}

	int projection = 0;

	if (strcasecmp(argv[2], "xt") == 0)
	{
		projection = 1;
	}
	else if (strcasecmp(argv[2], "yt") == 0)
	{
		projection = 2;
	}
	else if (strcasecmp(argv[2], "xy") == 0)
	{
		projection = 3;
	}

	FILE *f = fopen(argv[1], "r");
	if (f == NULL)
	{
		cerr << "Cannot open " << argv[1] << "." << endl;
		return -1;
	}

	int id, t1, t2;
	float x1, x2, y1, y2;
	char buf[1000];
	fgets(buf, 1000, f);

	while (! feof(f))
	{
		sscanf(buf, "%d %f %f %f %f %d %d\n", &id, &x1, &x2, &y1, &y2, &t1, &t2);

		if (projection == 0)
		{
			cout << x1 << " " << y1 << " " << t1 << endl;
			cout << x2 << " " << y1 << " " << t1 << endl;
			cout << x2 << " " << y2 << " " << t1 << endl;
			cout << x1 << " " << y2 << " " << t1 << endl;
			cout << x1 << " " << y1 << " " << t1 << endl;
			cout << x1 << " " << y1 << " " << t2 << endl;
			cout << x2 << " " << y1 << " " << t2 << endl;
			cout << x2 << " " << y2 << " " << t2 << endl;
			cout << x1 << " " << y2 << " " << t2 << endl;
			cout << x1 << " " << y1 << " " << t2 << endl;
			cout << x2 << " " << y1 << " " << t2 << endl;
			cout << x2 << " " << y1 << " " << t1 << endl;
			cout << x2 << " " << y2 << " " << t1 << endl;
			cout << x2 << " " << y2 << " " << t2 << endl;
			cout << x1 << " " << y2 << " " << t2 << endl;
			cout << x1 << " " << y2 << " " << t1 << endl;
			cout << endl << endl;
		}
		else if (projection == 1)
		{
			cout << x1 << " " << t1 << endl;
			cout << x2 << " " << t1 << endl;
			cout << x2 << " " << t2 << endl;
			cout << x1 << " " << t2 << endl;
			cout << x1 << " " << t1 << endl;
			cout << endl << endl;
		}
		else if (projection == 2)
		{
			cout << y1 << " " << t1 << endl;
			cout << y2 << " " << t1 << endl;
			cout << y2 << " " << t2 << endl;
			cout << y1 << " " << t2 << endl;
			cout << y1 << " " << t1 << endl;
			cout << endl << endl;
		}
		else
		{
			cout << x1 << " " << y1 << endl;
			cout << x2 << " " << y1 << endl;
			cout << x2 << " " << y2 << endl;
			cout << x1 << " " << y2 << endl;
			cout << x1 << " " << y1 << endl;
			cout << endl << endl;
		}

		fgets(buf, 1000, f);
	}

	fclose(f);
	return 0;
}
