// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Converts boxes to spatial index library R-tree format.
*/

#include "global.h"

int main(int argc, char *argv[])
{
	if (argc != 4) {
		cout << "Usage: " << argv[0] << " input_file output_file data_type\n";
		std::exit(-1);
	}

	int output = creat(argv[2], 0644);
	if (output == -1) {
		cerr << "Cannot open output \n";
		std::exit(-1);
	}

	FILE *input = fopen(argv[1], "r");
	if (input == NULL) {
		cerr << "Cannot open input \n";
		std::exit(-1);
	}

	int type = 0;
	if (0 == strcmp(argv[3], "query")) type = QUERY;
	else if (0 == strcmp(argv[3], "data")) type = INSERT;

	int count = 0, oid;
	float x1, x2, y1, y2, start, end;
	char buf[1000];
	fgets(buf, 1000, input);

	while (!feof(input))
	{
		sscanf(buf, "%d %f %f %f %f %f %f\n", &oid, &x1, &x2, &y1, &y2, &start, &end);
		write(output, &type, sizeof(int));
		write(output, &x1, sizeof(float));
		write(output, &x2, sizeof(float));
		write(output, &y1, sizeof(float));
		write(output, &y2, sizeof(float));
		write(output, &start, sizeof(float));
		write(output, &end, sizeof(float));

		fgets(buf, 1000, input);
		count++;
	}

	int i = -1;
	write(output, &i, sizeof(int));

	cerr << "Inserted " << count << " rectangles\n";
	close(output);
	fclose(input);
	return 0;
}


