// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Converts generator data to gnuplot format for visualization purposes.
*/

#include "global.h"

int main(int argc, char** argv)
{
	if (argc != 3)
	{
		cerr << "Usage: " << argv[0] << " data_file projection (xt | yt | xy | xyt)." << endl;
		return -1;
	}

	ifstream f(argv[1]);
	if (! f)
	{
		cerr << "Cannot open file " << argv[1] << "." << endl;
		return -1;
	}
	
	cout.precision(PRECISION);
	cout << fixed;

	MovingObject o;
	while (parseObject(f, o))
	{
		vector<Point> c;
		o.toPointArray(c);

		for (unsigned int i = 0; i < c.size(); i++)
		{
			if (strcasecmp(argv[2], "xt") == 0)
			{
				cout << c[i].m_x << " " << c[i].m_t << endl;
			}
			else if (strcasecmp(argv[2], "yt") == 0)
			{
				cout << c[i].m_y << " " << c[i].m_t << endl;
			}
			else if (strcasecmp(argv[2], "xy") == 0)
			{
				cout << c[i].m_x << " " << c[i].m_y << endl;
			}
			else
			{
				cout << c[i].m_x << " " << c[i].m_y << " " << c[i].m_t << endl;
			}
		}
		cout << endl << endl;
	}
}
