// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

/*
	Converts gstd data format to generator format.
*/

#include "global.h"

int main(int argc, char** argv)
{
	if (argc != 2)
	{
		cerr << "Usage: " << argv[0] << " input_file." << endl;
		return -1;
	}

	FILE *f = fopen(argv[1], "r");
	if (f == NULL)
	{
		cerr << "Cannot open " << argv[1] << "." << endl;
		return -1;
	}

	int t = -1, id, unused;
	long count = 0;
	float fl, x1, x2, y1, y2;
	MovingObject* o;
	map<int, Point> points;
	map<int, MovingObject*> dataset;
	map<int, MovingObject*>::iterator it;

	while (! feof(f))
	{
		fscanf(f, "%d %f", &id, &fl);
		if (feof(f)) break;

		t = (int) ceil(fl * 100.0);

		fscanf(f, "%f %f %f %f %d\n", &x1, &y1, &x2, &y2, &unused);
		Point p2 = Point(t, x1, y1);

		map<int, Point>::iterator itPoint = points.find(id);
		if (itPoint != points.end())
		{
			Point p1 = (*itPoint).second;

			if (p2.m_t == p1.m_t) p2.m_t++;

			double aCoef[2];
			aCoef[1] = (p2.m_x - p1.m_x) / (p2.m_t - p1.m_t);
			aCoef[0] = p2.m_x - (aCoef[1] * p2.m_t);
			Polynomial px = Polynomial(2, aCoef);

			aCoef[1] = (p2.m_y - p1.m_y) / (p2.m_t - p1.m_t);
			aCoef[0] = p2.m_y - (aCoef[1] * p2.m_t);
			Polynomial py = Polynomial(2, aCoef);

			it = dataset.find(id);
			if (it == dataset.end())
			{
				o = new MovingObject();
				o->m_id = id;
				o->m_functions.push_back(Function(p1.m_t, p2.m_t, px, py));
				dataset.insert(pair<int, MovingObject*>(id, o));
			}
			else
			{
				(*it).second->m_functions.push_back(Function(p1.m_t, p2.m_t, px, py));
			}
		}

		points.erase(id);
		points.insert(pair<int, Point>(id, p2));

		if ((count % 10000) == 0) cerr << count << endl;
		count++;
	}

	//cout << dataset.size() << endl;

	for (it = dataset.begin(); it != dataset.end(); it++)
	{
		MovingObject* o = (*it).second;

		for (unsigned int cIndex = 0; cIndex < o->m_functions.size(); cIndex++)
		{
			int tend = (o->m_functions[cIndex].m_end == -1) ? t : o->m_functions[cIndex].m_end;
			cout << o->m_id << " " << o->m_functions[cIndex].m_start << " "  << tend << " " << o->m_functions[cIndex].m_x.m_cLength << " ";
			for (int cCoef = 0; cCoef < o->m_functions[cIndex].m_x.m_cLength; cCoef++)
			{
				cout << o->m_functions[cIndex].m_x.m_pCoefficients[cCoef] << " ";
			}

			cout << o->m_functions[cIndex].m_y.m_cLength << " ";
			for (int cCoef = 0; cCoef < o->m_functions[cIndex].m_y.m_cLength; cCoef++)
			{
				cout << o->m_functions[cIndex].m_y.m_pCoefficients[cCoef] << " ";
			}

			cout << "0 0" << endl;
		}

		cout << endl;
	}

	return 0;
}
