// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "global.h"

int main(int argc, char** argv)
{
	if (argc != 2)
	{
		cerr << "Usage: " << argv[0] << " input_file." << endl;
		return -1;
	}

	FILE *f = fopen(argv[1], "r");
	if (f == NULL)
	{
		cerr << "Cannot open " << argv[1] << "." << endl;
		return -1;
	}

	int T = -1;
	long count = 0;
	int id, op, ft, degree;
	float fl;
	double* pCoefficients;
	char buf[1000];
	MovingObject* o;
	map<int, MovingObject*> dataset;
	map<int, MovingObject*>::iterator it;
	vector<MovingObject*> finalobjects;

	while (! feof(f))
	{
		fscanf(f, "%d %d %d", &id, &op, &T);
		//cerr << id << " " << op << " " << T << " " << endl;
		if (feof(f)) break;

		if (op == INSERT)
		{
			fscanf(f, "%d %d", &ft, &degree);
			pCoefficients = new double[degree + 1];
			for (int cCoefficient = 0; cCoefficient <= degree; cCoefficient++)
			{
				fscanf(f, "%f", &fl);
				pCoefficients[cCoefficient] = fl;
			}
			Polynomial px(degree + 1, pCoefficients);
			delete[] pCoefficients;

			fscanf(f, "%d", &degree);
			pCoefficients = new double[degree + 1];
			for (int cCoefficient = 0; cCoefficient <= degree; cCoefficient++)
			{
				fscanf(f, "%f", &fl);
				pCoefficients[cCoefficient] = fl;
			}
			Polynomial py(degree + 1, pCoefficients);
			delete[] pCoefficients;

			it = dataset.find(id);
			if (it == dataset.end())
			{
				o = new MovingObject();
				o->m_id = id;
				o->m_functions.push_back(Function(ft, -1, px, py));
				dataset.insert(pair<int, MovingObject*>(id, o));
			}
			else
			{
				Function& f = (*it).second->m_functions.back();
				cerr << f.m_end << " " << f.m_x.getValue(f.m_end) << " " << ft << " " << px.getValue(ft) << endl;
				cerr << f.m_end << " " << f.m_y.getValue(f.m_end) << " " << ft << " " << py.getValue(ft) << endl;

				if (f.m_end != ft ||
						f.m_x.getValue(f.m_end) < px.getValue(ft) - std::numeric_limits<double>::epsilon() ||
					 	f.m_x.getValue(f.m_end) > px.getValue(ft) + std::numeric_limits<double>::epsilon() ||
						f.m_y.getValue(f.m_end) < py.getValue(ft) - std::numeric_limits<double>::epsilon() ||
					 	f.m_y.getValue(f.m_end) > py.getValue(ft) + std::numeric_limits<double>::epsilon())
				{
					// the object was deleted in the past and has reappeared or it has made a jump.
					// Store the first instance in the finalobjects array
					// and replace it in the dataset with the new instance.

					finalobjects.push_back((*it).second);

					o = new MovingObject();
					o->m_id = id;
					o->m_functions.push_back(Function(ft, -1, px, py));
					dataset.erase(it);
					dataset.insert(pair<int, MovingObject*>(id, o));
				}
				else
				{
					(*it).second->m_functions.push_back(Function(ft, -1, px, py));
				}
			}
		}
		else if (op == DELETE)
		{
			it = dataset.find(id);
			if (it == dataset.end())
			{
				cerr << "Imposible! (count=" << count << ")" << endl;
				return -1;
			}
			else
			{
				Function& f = (*it).second->m_functions.back();
				f.m_end = T;
			}
		}

		fgets(buf, 1000, f);

		if ((count % 10000) == 0) cerr << "Count: " << count << endl;
		count++;
	}

	for (it = dataset.begin(); it != dataset.end(); it++)
	{
		MovingObject* o = (*it).second;
		Function& f = o->m_functions.back();
		if (f.m_start == T) o->m_functions.pop_back();
		else f.m_end = T;
		if (! o->m_functions.empty())
		{
			finalobjects.push_back(o);
		}
	}

	//cout << finalobjects.size() << endl;

	for (unsigned int cID = 0; cID < finalobjects.size(); cID++)
	{
		MovingObject* o = finalobjects[cID];

		for (unsigned int cIndex = 0; cIndex < o->m_functions.size(); cIndex++)
		{
			cout << cID << " "
				<< o->m_functions[cIndex].m_start << " "  << o->m_functions[cIndex].m_end << " "
				<< o->m_functions[cIndex].m_x.m_cLength << " ";

			for (int cCoef = 0; cCoef < o->m_functions[cIndex].m_x.m_cLength; cCoef++)
			{
				cout << o->m_functions[cIndex].m_x.m_pCoefficients[cCoef] << " ";
			}

			cout << o->m_functions[cIndex].m_y.m_cLength << " ";
			for (int cCoef = 0; cCoef < o->m_functions[cIndex].m_y.m_cLength; cCoef++)
			{
				cout << o->m_functions[cIndex].m_y.m_pCoefficients[cCoef] << " ";
			}

			cout << "0 0" << endl;
		}

		cout << endl;
		delete o;
	}

	return 0;
}
