// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "global.h"

using namespace std;

int main(int argc, char** argv)
{
	if (argc != 4)
	{
		cerr << "Usage: " << argv[0] << " data_file volume_threshold lifetime_threshold." << endl;
		return -1;
	}

	double vt = atof(argv[2]);
	double lt = atof(argv[3]);

	ifstream df(argv[1]);
	if (! df)
	{
		cerr << "Cannot open " << argv[1] << "." << endl;
		return -1;
	}

	cout.precision(PRECISION);
	cout << fixed;

	MovingObject o;
	unsigned long totalObjects = 0;
	unsigned long qualifyingObjects = 0;

	while(parseObject(df, o))
	{
		totalObjects++;

		if ((double) (o.m_functions[o.m_functions.size() - 1].m_end - o.m_functions[0].m_start) > lt ||
			o.getVolume() > vt)
		{
			qualifyingObjects++;
			cout << o << endl;
		}
	}

	cerr << "Total objects: " << totalObjects << endl;
	cerr << "Qualifying objects: " << qualifyingObjects << endl;

	return 0;
}

