// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "global.h"
#include <iostream>
#include <fstream>
#include <map>
#include <queue>

using namespace std;

class NNEntry
{
public:
	unsigned long m_id;
	double m_dist;

	NNEntry(unsigned long id, double dist) : m_id(id), m_dist(dist) {}

	struct greater : public binary_function<NNEntry*, NNEntry*, bool>
	{
		bool operator()(const NNEntry* __x, const NNEntry* __y) const { return __x->m_dist > __y->m_dist; }
	};
};

int main(int argc, char** argv)
{
	if (argc != 3)
	{
		cerr << "Usage: " << argv[0] << " data_file query_type [intersection | 10NN]." << endl;
		return -1;
	}

	int queryType = 0;

	if (strcmp(argv[2], "intersection") == 0) queryType = 0;
	else if (strcmp(argv[2], "10NN") == 0) queryType = 1;
	else
	{
		cerr << "Unknown query type." << endl;
		return -1;
	}

	ifstream fin(argv[1]);
	if (! fin)
	{
		cerr << "Cannot open data file" << argv[1] << "." << endl;
		return -1;
	}

	multimap<unsigned long, Rectangle> data;
	unsigned long id, op;
	double x1, x2, y1, y2, t, t1, t2;

	while (fin)
	{
		fin >> t >> op >> id >> x1 >> x2 >> y1 >> y2 >> t1 >> t2;
		if (! fin.good()) continue;

		if (op == INSERT)
		{
			//insert
			//data.insert(pair<unsigned long, Rectangle>(id, Rectangle(x1, x2, y1, y2, t, std::numeric_limits<double>::max())));
			// allow multiple entries with the same id
			data.insert(pair<unsigned long, Rectangle>(id, Rectangle(x1, x2, y1, y2, t, std::numeric_limits<double>::max())));
		}
		else if (op == DELETE)
		{
			//delete
			// find the live instance of id.
			map<unsigned long, Rectangle>::iterator it = data.find(id);
			assert(it != data.end());
			while (it->first == id && it->second.m_thigh < std::numeric_limits<double>::max()) it++;
			assert(it->first == id);
			(*it).second.m_thigh = t;
		}
		else if (op == QUERY)
		{
			//query
			if (queryType == 0)
			{
				Rectangle query = Rectangle(x1, x2, y1, y2, t1, t2);
				for (map<unsigned long, Rectangle>::iterator it = data.begin(); it != data.end(); it++)
				{
					if (query.intersectsInTime((*it).second)) cout << (*it).first << endl;
				}
			}
			else
			{
				/*
				Rectangle query = Rectangle(x1, x2, y1, y2, qt1, qt2);

				priority_queue<NNEntry*, vector<NNEntry*>, NNEntry::greater > queue;

				for (map<long, Region>::iterator it = data.begin(); it != data.end(); it++)
				{
					queue.push(new NNEntry((*it).first, (*it).second.getMinDist(query)));
				}

				int count = 0;
				double knearest = 0.0;

				while (! queue.empty())
				{
					NNEntry* e = queue.top(); queue.pop();

					if (count >= 10 && e->m_dist > knearest) break;

					//cout << e->m_id << " " << e->m_dist << endl;
					cout << e->m_id << endl;
					count++;
					knearest = e->m_dist;
					delete e;
				}

				while (! queue.empty())
				{
					NNEntry* e = queue.top(); queue.pop();
					delete e;
				}
			*/
			}
		}
	}

	return 0;
}
