// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#ifndef _attractor_generator_h
#define _attractor_generator_h

#include "Generator.h"

class AttractorGenerator : public Generator
{
public:
	int m_cAttractors;
	double m_attractionProbability;
	bool m_changingAttractors;
	int m_changeInterval;
	Point* m_attractors;

	AttractorGenerator(int ds, int sl, int mui, double a, int an, double ap, bool c, int ct)
		: Generator(ds, sl, mui, a),
			m_cAttractors(an),
			m_attractionProbability(ap),
			m_changingAttractors(c),
			m_changeInterval(ct),
			m_attractors(0)
	{
		m_updateArray = new set<int>[m_simulationLength];
		createRandomAttractors();
	}

	virtual ~AttractorGenerator() { delete[] m_updateArray; }

	class MyMovingObject : public MovingObject
	{
	public:
		int m_attractorIndex;
		bool m_towardsAttractor;
		bool m_willReachAttractor;
		bool m_outOfBounds;
		double m_sx, m_sy;
		double m_vx, m_vy;
		vector<Function> m_functions;

		double getX(int t)
		{
			return m_sx + m_vx * (t - m_st);
		}

		double getY(int t)
		{
			return m_sy + m_vy * (t - m_st);
		}
	};

	virtual void generate();
	virtual void adjustSpeed(MyMovingObject* o);

	virtual void createRandomAttractors();

};

#endif

