// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#ifndef _generator_h
#define _generator_h

#include "global.h"

class Generator
{
public:
  int m_datasetSize;
  int m_simulationLength;
  int m_speedDistribution;
  int m_initialDistribution;
  int m_maximumUpdateInterval;
  double m_agility;
  double m_minSpeed;
  double m_maxSpeed;
  double m_speedMean;
  double m_speedStandardDeviation;
  double m_maxX;
  double m_maxY;

  map<int, MovingObject*> m_dataset;
  set<int>* m_updateArray;

  Generator(int ds, int sl, int mui, double a)
    : m_datasetSize(ds),
      m_simulationLength(sl),
      m_speedDistribution(ZIPF),
      m_initialDistribution(UNIFORM),
      m_maximumUpdateInterval(mui),
      m_agility(a),
      m_minSpeed(0.0025), // 15 miles/hour = 0.25 miles/minute
      m_maxSpeed(0.0166), // 100 miles/hour = 1.66 miles/minute
      m_speedMean(0.005), // 30 miles/hour = 0.5 miles/minute
      m_speedStandardDeviation(0.0033), // 20 miles/hour = 0.33 miles/minute
      m_maxX(XMAX), 
      m_maxY(YMAX),
      m_updateArray(0)
  {
    initRandom();
  }

  virtual ~Generator()
  {
    for (map<int, MovingObject*>::iterator it = m_dataset.begin(); it != m_dataset.end(); it++)
    {
      delete (*it).second;
    }
  }

  virtual void generate() = 0;

  double generateSpeed()
  {
    if (m_speedDistribution == ZIPF)
    {
      return nextZipfDouble(m_minSpeed, m_maxSpeed, 1.0);
    }
    else if (m_speedDistribution == GAUSSIAN)
    {
      double v = nextGaussianDouble(m_speedMean, m_speedStandardDeviation);
      if (v > m_maxSpeed) v = m_maxSpeed;
      else if (v < m_minSpeed) v = m_minSpeed;
      return v;
    }
    else
    {
      return nextUniformDouble(m_minSpeed, m_maxSpeed);
    }
  }
};

#endif

