// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#ifndef _polynomial_generator_h
#define _polynomial_generator_h

#include "Generator.h"

class PolynomialGenerator : public Generator
{
public:
  int m_lastId;

public:
	PolynomialGenerator(int ds, int sl, int mui, double a)
		: Generator(ds, sl, mui, a)
	{
		m_updateArray = new set<int>[m_simulationLength];
	}

	virtual ~PolynomialGenerator() { delete[] m_updateArray; }

	class MyMovingObject : public MovingObject
	{
	public:
		bool m_outOfBounds;
		vector<Function> m_functions;
	  Polynomial m_px;
  	Polynomial m_py;

	  MyMovingObject() {}
  	MyMovingObject(int id, Polynomial& px, Polynomial& py, int st)
    : m_px(px), m_py(py)
    {
			m_id = id;
			m_st = st;
    }

	  double getX(int t)
  	{
    	// the poynomials are already relative to m_st.
    	return m_px.getValue((double) (t));
  	}

  	double getY(int t)
	  {
	  	// the poynomials are already relative to m_st.
  	  return m_py.getValue((double) (t));
	  }
	};

	virtual void generate();
	Polynomial createPolynomial(double start, unsigned int st);
	MovingObject* createObject(int id, int st, double xmin, double xmax, double ymin, double ymax, int dist, bool bKilled);
	MovingObject* createObject(int id, int st, double x, double y, bool bKilled);
  void printObject(MyMovingObject* o, bool bKilled);
};

#endif

