// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "AttractorGenerator.h"

int main(int argc, char** argv)
{
	int ds = 1000, sl = TMAX, mui = TMAX + 10, an = 5;
	double a = 0.01;
	double ap = 0.7;
	int sd = GAUSSIAN, id = ZIPF;

	for (int i = 1; i < argc; i++)
	{
		if (! strcmp(argv[i], "-ds"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing dataset size." << endl;
				return -1;
			}
			ds = atoi(argv[i]);
		}
		else if (! strcmp(argv[i], "-sl"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing simulation length." << endl;
				return -1;
			}
			sl = atoi(argv[i]);
		}
		else if (! strcmp(argv[i], "-sd"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing speed distribution." << endl;
				return -1;
			}
			if (strcmp(argv[i], "zipf") == 0) sd = ZIPF;
			else if (strcmp(argv[i], "gaussian") == 0) sd = GAUSSIAN;
			else sd = UNIFORM;
		}
		else if (! strcmp(argv[i], "-id"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing initial distribution." << endl;
				return -1;
			}
			if (strcmp(argv[i], "zipf") == 0) id = ZIPF;
			else if (strcmp(argv[i], "gaussian") == 0) id = GAUSSIAN;
			else id = UNIFORM;
		}
		else if (! strcmp(argv[i], "-a"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing agility." << endl;
				return -1;
			}
			a = atof(argv[i]);
		}
		else if (! strcmp(argv[i], "-mui"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing update rate." << endl;
				return -1;
			}
			mui = atoi(argv[i]);
		}
		else if (! strcmp(argv[i], "-an"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing attractor number." << endl;
				return -1;
			}
			an = atoi(argv[i]);
		}
		else if (! strcmp(argv[i], "-ap"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing attraction probability." << endl;
				return -1;
			}
			ap = atof(argv[i]);
		}
		else
		{
			cerr << "Usage: " << endl
					 << "  -ds dataset size" << endl
					 << "  -sl simulation length" << endl
					 << "  -a agility" << endl
					 << "  -sd speed distribution" << endl
					 << "  -id initial distribution" << endl
					 << "  -an attractor number" << endl
					 << "  -ap attraction probability" << endl
					 << "  -mui maximum update interval" << endl;
			return -1;
		}
	}

	AttractorGenerator g = AttractorGenerator(ds, sl, mui, a, an, ap, false, 0);
	g.m_speedDistribution = sd;
	g.m_initialDistribution = id;
	g.generate();
}

