// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "global.h"

int main(int argc, char** argv)
{
  int ds = 1000, sl = TMAX;
  double a = 0.01;
  double xmax = XMAX;
  double ymax = YMAX;
  int dt = 2;

  for (int i = 1; i < argc; i++)
  {
    if (! strcmp(argv[i], "-ds"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing dataset size." << endl;
        return -1;
      }
      ds = atoi(argv[i]);
    }
    else if (! strcmp(argv[i], "-xmax"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing xmax." << endl;
        return -1;
      }
      xmax= atof(argv[i]);
    }
    else if (! strcmp(argv[i], "-ymax"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing ymax." << endl;
        return -1;
      }
      ymax= atof(argv[i]);
    }
    else if (! strcmp(argv[i], "-sl"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing simulation length." << endl;
        return -1;
      }
      sl = atoi(argv[i]);
    }
    else if (! strcmp(argv[i], "-dt"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing dt." << endl;
        return -1;
      }
      dt = atoi(argv[i]);
    }
    else if (! strcmp(argv[i], "-a"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing alpha." << endl;
        return -1;
      }
      a = atof(argv[i]);
    }
    else
    {
      cerr << "Usage: " << endl
           << "  -ds dataset size" << endl
           << "  -sl simulation length" << endl
           << "  -dt query time interval" << endl
           << "  -a query extent" << endl
           << "  -xmax x universe extent" << endl
           << "  -ymax y universe extent" << endl;
      return -1;
    }
  }

  initRandom();

  double dx = a * xmax / 2.0;
  double dy = a * ymax / 2.0;

	cout.precision(8);
	cout << fixed;

	for (int cCount = 0; cCount < ds; cCount++)
	{
    double x = nextUniformDouble(0.0, xmax);
		double y = nextUniformDouble(0.0, ymax);
  	int t = nextUniformInt(0, sl);

		cout << cCount << " " << ((x - dx < 0.0) ? 0.0 : x - dx) << " " << ((x + dx > xmax) ? xmax : x + dx) << " "
			<< ((y- dy < 0.0) ? 0.0 : y - dy) << " " << ((y + dy > ymax) ? ymax : y + dy) << " " << t << " " << std::min(t + dt - 1, sl) << endl;
	}

  return 0;
}
