// Efficient Indexing of Spatiotemporal Objects
// Copyright (C) 2003 Marios Hadjieleftheriou
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Contact information:
//  Mailing address:
//    Marios Hadjieleftheriou
//    University of California, Riverside
//    Department of Computer Science
//    Surge Building, Room 310
//    Riverside, CA 92521
//
//  Email:
//    marioh@cs.ucr.edu

#include "RoadGenerator.h"

int main(int argc, char** argv)
{
  int ds = 30, sl = TMAX, mui = TMAX + 10, sd = GAUSSIAN, id = UNIFORM;
  double a = 0.01;
  string s = "road1.txt";

  for (int i = 1; i < argc; i++)
  {
    if (! strcmp(argv[i], "-ds"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing dataset size." << endl;
        return -1;
      }
      ds = atoi(argv[i]);
    }
    else if (! strcmp(argv[i], "-rf"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing road file." << endl;
        return -1;
      }
      s = argv[i];
    }
    else if (! strcmp(argv[i], "-sl"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing simulation length." << endl;
        return -1;
      }
      sl = atoi(argv[i]);
    }
		else if (! strcmp(argv[i], "-sd"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing speed distribution." << endl;
				return -1;
			}
      if (strcmp(argv[i], "zipf") == 0) sd = ZIPF;
      else if (strcmp(argv[i], "gaussian") == 0) sd = GAUSSIAN;
      else sd = UNIFORM;
		}
		else if (! strcmp(argv[i], "-id"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing initial distribution." << endl;
				return -1;
			}
      if (strcmp(argv[i], "zipf") == 0) id = ZIPF;
      else if (strcmp(argv[i], "gaussian") == 0) id = GAUSSIAN;
      else id = UNIFORM;
		}
    else if (! strcmp(argv[i], "-a"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing agility." << endl;
        return -1;
      }
      a = atof(argv[i]);
    }
    else if (! strcmp(argv[i], "-mui"))
    {
      i++;
      if (i >= argc)
      {
        cerr << "Missing update rate." << endl;
        return -1;
      }
      mui = atoi(argv[i]);
    }
    else
    {
      cerr << "Usage: " << endl
           << "  -ds dataset size" << endl
           << "  -rf road file" << endl
           << "  -sl simulation length" << endl
           << "  -a agility" << endl
           << "  -sd speed distribution" << endl
           << "  -id initial distribution" << endl
           << "  -mui maximum update interval" << endl;
      return -1;
    }
  }

  RoadGenerator g = RoadGenerator(ds, sl, mui, a, s);
  g.m_speedDistribution = sd;
  g.m_initialDistribution = id;
  g.generate();
}

