#include "global.h"

int main(int argc, char** argv)
{
	char dataFile[1000];
	int policy = GREEDY_SPLIT;

	for (int i = 1; i < argc; i++)
	{
		if (! strcmp(argv[i], "-df"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing data file." << endl;
				return -1;
			}
			strncpy(dataFile, argv[i], 1000);
		}
		else if (! strcmp(argv[i], "-p"))
		{
			i++;
			if (i >= argc)
			{
				cerr << "Missing policy." << endl;
				return -1;
			}
			if (strcasecmp(argv[i], "dynamic") == 0) policy = DYNAMIC_SPLIT;
			else if (strcasecmp(argv[i], "greedy") == 0) policy = GREEDY_SPLIT;
		}
		else
		{
			cerr << "Usage: " << endl
					 << "  -df data file" << endl
					 << "  -p policy (dynamic | greedy)" << endl;
			return -1;
		}
	}

	ifstream f(dataFile);
	if (! f)
	{
		cerr << "Cannot open " << dataFile << "." << endl;
		return -1;
	}

	cerr << "Policy: " << policy << endl;
	cout.precision(PRECISION);
	cout << fixed;

	cpu_usage_aid cpu;
	begin_cpu_usage(&cpu);

	MovingObject o;
	while (parseObject(f, o))
	{
		cerr << o.m_id << endl;

		SplitInformation si;
		if (policy == GREEDY_SPLIT) greedySplit(o, si);
		else if (policy == DYNAMIC_SPLIT) dynamicSplit(o, si);

		if (si.getTotalSplits() == 0)
		{
			cout << o.m_id << "\t0\t0\t" << si.m_volumes[0] << endl << endl;
		}
		else
		{
			for (unsigned int cSplit = 0; cSplit < si.m_volumes.size(); cSplit++)
			{
				cout << si.m_id << "\t" << si.getTotalSplits() << "\t" << cSplit << "\t" << si.m_volumes[cSplit] << "\t";

        if (cSplit <= MAXIMUM_SPLITS_KEPT)
        {
  				for (unsigned int cIndex = 0; cIndex < cSplit; cIndex++)
	  			{
		  			cout << si.m_splitPoints[cSplit][cIndex] << " ";
  				}
        }
	  		cout << endl;
      }
			cout << endl;
		}
	}

	cpu_usage cpuu;
	end_cpu_usage(&cpu, &cpuu);

	cerr << "Total time: " << cpuu.total_time << endl;
	cerr << "System time: " << cpuu.system_time << endl;
	cerr << "User time: " << cpuu.user_time << endl;

	return 0;
}
